searchData={"items":[{"type":"module","title":"Owl.Box","doc":"Allows wrapping data to boxes.","ref":"Owl.Box.html"},{"type":"function","title":"Owl.Box.new/2","doc":"Wraps data into a box.","ref":"Owl.Box.html#new/2"},{"type":"function","title":"Options - Owl.Box.new/2","doc":"* `:padding` - sets the padding area for all four sides at once.  Defaults to 0.\n* `:padding_x` - sets `:padding_right` and `:padding_left` at once. Overrides value set by `:padding`. Defaults to 0.\n* `:padding_y` - sets `:padding_top` and `:padding_bottom` at once. Overrides value set by `:padding`. Defaults to 0.\n* `:padding_top` - sets the padding area for top side. Overrides value set by `:padding_y` or `:padding`.  Defaults to 0.\n* `:padding_bottom` - sets the padding area for bottom side. Overrides value set by `:padding_y` or `:padding`. Defaults to 0.\n* `:padding_right` - sets the padding area for right side. Overrides value set by `:padding_x` or `:padding`. Defaults to 0.\n* `:padding_left` - sets the padding area for left side. Overrides value set by `:padding_x` or `:padding`. Defaults to 0.\n* `:min_height` - sets the minimum height of the box, including paddings and size of the borders. Defaults to 0.\n* `:min_width` - sets the minimum width of the box, including paddings and size of the borders. Defaults to 0.\n* `:max_width` - sets the maximum width of the box, including paddings and size of the borders. Defaults to width of the terminal, if available, `:infinity` otherwise.\n* `:horizontal_align` - sets the horizontal alignment of the content inside a box. Defaults to `:right`.\n* `:vertical_align` - sets the vertical alignment of the content inside a box. Defaults to `:top`.\n* `:border_style` - sets the border style. Defaults to `:solid`.\n* `:border_tag` - sets the tag for border characters. See `t:Owl.Data.sequence/0` for a valid sequences Defaults to `[]`.\n* `:title` - sets a title that is displayed in a top border. Ignored if `:border_style` is `:none`. Defaults to `nil`.\n* `:word_wrap` - sets the word wrapping mode. Can be `:break_word` or `:normal`. Defaults to `:break_word`. Ignored if `:truncate_lines` is `true`.\n* `:truncate_lines` - specifies whether to truncate lines that are too long to fit into a box. Defaults to `false`.","ref":"Owl.Box.html#new/2-options"},{"type":"function","title":"Examples - Owl.Box.new/2","doc":"iex> \"Owl\" |> Owl.Box.new() |> to_string()\n    \"\"\"\n    ┌───┐\n    │Owl│\n    └───┘\n    \"\"\" |> String.trim_trailing()\n\n    iex> \"Owl\" |> Owl.Box.new(padding_x: 4) |> to_string()\n    \"\"\"\n    ┌───────────┐\n    │    Owl    │\n    └───────────┘\n    \"\"\" |> String.trim_trailing()\n\n    iex> \"Hello\\nworld!\"\n    ...> |> Owl.Box.new(\n    ...>   title: \"Greeting!\",\n    ...>   min_width: 20,\n    ...>   horizontal_align: :center,\n    ...>   border_style: :double\n    ...> )\n    ...> |> to_string()\n    \"\"\"\n    ╔═Greeting!════════╗\n    ║      Hello       ║\n    ║      world!      ║\n    ╚══════════════════╝\n    \"\"\" |> String.trim_trailing()\n\n    iex> \"Success\"\n    ...> |> Owl.Box.new(\n    ...>   min_width: 20,\n    ...>   min_height: 3,\n    ...>   border_style: :none,\n    ...>   horizontal_align: :right,\n    ...>   vertical_align: :bottom\n    ...> )\n    ...> |> to_string()\n    \"\"\"\n                        \n                        \n                 Success\n    \"\"\" |> String.trim_trailing()\n\n    iex> \"OK\"\n    ...> |> Owl.Box.new(min_height: 5, vertical_align: :middle)\n    ...> |> to_string()\n    \"\"\"\n    ┌──┐\n    │  │\n    │OK│\n    │  │\n    └──┘\n    \"\"\" |> String.trim_trailing()\n\n    iex> \"VeryLongLine\" |> Owl.Box.new(max_width: 6) |> to_string()\n    \"\"\"\n    ┌────┐\n    │Very│\n    │Long│\n    │Line│\n    └────┘\n    \"\"\" |> String.trim_trailing()\n\n    iex> \"VeryLongLine\" |> Owl.Box.new(max_width: 4, border_style: :none) |> to_string()\n    \"\"\"\n    Very\n    Long\n    Line\n    \"\"\" |> String.trim_trailing()\n\n    iex> \"Green!\"\n    ...> |> Owl.Data.tag(:green)\n    ...> |> Owl.Box.new(title: Owl.Data.tag(\"Red!\", :red))\n    ...> |> Owl.Data.tag(:cyan)\n    ...> |> Owl.Data.to_chardata()\n    ...> |> to_string()\n    \"\"\"\n    \\e[36m┌─\\e[31mRed!\\e[36m────┐\\e[39m\n    \\e[36m│\\e[32mGreen!\\e[36m   │\\e[39m\n    \\e[36m└─────────┘\\e[39m\\e[0m\n    \"\"\" |> String.trim_trailing()\n\n    iex> \"Green!\"\n    ...> |> Owl.Data.tag(:green)\n    ...> |> Owl.Box.new(title: Owl.Data.tag(\"Red!\", :red))\n    ...> |> Owl.Data.tag(:cyan)\n    ...> |> Owl.Data.to_chardata()\n    ...> |> to_string()\n    \"\"\"\n    \\e[36m┌─\\e[31mRed!\\e[36m────┐\\e[39m\n    \\e[36m│\\e[32mGreen!\\e[36m   │\\e[39m\n    \\e[36m└─────────┘\\e[39m\\e[0m\n    \"\"\" |> String.trim_trailing()\n\n    iex> \"Hello\\nworld!\"\n    ...> |> Owl.Box.new(\n    ...>   min_width: 20,\n    ...>   horizontal_align: :center,\n    ...>   border_style: :double,\n    ...>   border_tag: :cyan\n    ...> )\n    ...> |> Owl.Data.to_chardata()\n    ...> |> to_string()\n    \"\"\"\n    \\e[36m╔\\e[39m\\e[36m══════════════════\\e[39m\\e[36m╗\\e[39m\n    \\e[36m║\\e[39m      Hello       \\e[36m║\\e[39m\n    \\e[36m║\\e[39m      world!      \\e[36m║\\e[39m\n    \\e[36m╚\\e[39m\\e[36m══════════════════\\e[39m\\e[36m╝\\e[39m\\e[0m\n    \"\"\" |> String.trim_trailing()\n\n    iex> \"Hello\\nworld!\"\n    ...> |> Owl.Box.new(\n    ...>   title: \"Greeting!\",\n    ...>   min_width: 20,\n    ...>   horizontal_align: :center,\n    ...>   border_style: :double,\n    ...>   border_tag: :cyan\n    ...> )\n    ...> |> Owl.Data.to_chardata()\n    ...> |> to_string()\n    \"\"\"\n    \\e[36m╔\\e[39m\\e[36m═\\e[39mGreeting!\\e[36m════════\\e[39m\\e[36m╗\\e[39m\n    \\e[36m║\\e[39m      Hello       \\e[36m║\\e[39m\n    \\e[36m║\\e[39m      world!      \\e[36m║\\e[39m\n    \\e[36m╚\\e[39m\\e[36m══════════════════\\e[39m\\e[36m╝\\e[39m\\e[0m\n    \"\"\" |> String.trim_trailing()","ref":"Owl.Box.html#new/2-examples"},{"type":"module","title":"Owl.Data","doc":"A set of functions for `t:chardata/0` with [tags](`Owl.Tag`).","ref":"Owl.Data.html"},{"type":"function","title":"Owl.Data.add_prefix/2","doc":"Adds a `prefix` before each line of the `data`.\n\nAn important feature is that styling of the data will be saved for each line.","ref":"Owl.Data.html#add_prefix/2"},{"type":"function","title":"Example - Owl.Data.add_prefix/2","doc":"iex> \"first\\nsecond\" |> Owl.Data.tag(:red) |> Owl.Data.add_prefix(Owl.Data.tag(\"test: \", :yellow))\n    [\n      [Owl.Data.tag(\"test: \", :yellow), Owl.Data.tag([\"first\"], :red)],\n      \"\\n\",\n      [Owl.Data.tag(\"test: \", :yellow), Owl.Data.tag([\"second\"], :red)]\n    ]","ref":"Owl.Data.html#add_prefix/2-example"},{"type":"function","title":"Owl.Data.chunk_every/2","doc":"Returns list of `t()` containing `count` elements each.","ref":"Owl.Data.html#chunk_every/2"},{"type":"function","title":"Example - Owl.Data.chunk_every/2","doc":"iex> Owl.Data.chunk_every(\n    ...>   [\"first second \", Owl.Data.tag([\"third\", Owl.Data.tag(\" fourth\", :blue)], :red)],\n    ...>   7\n    ...> )\n    [\n      \"first s\",\n      [\"econd \", Owl.Data.tag([\"t\"], :red)],\n      Owl.Data.tag([\"hird\", Owl.Data.tag([\" fo\"], :blue)], :red),\n      Owl.Data.tag([\"urth\"], :blue)\n    ]","ref":"Owl.Data.html#chunk_every/2-example"},{"type":"function","title":"Owl.Data.from_chardata/1","doc":"Transforms chardata, replacing raw escape sequences with tags (see `tag/2`).\n\nThis makes it possible to use data formatted outside of Owl with other Owl modules, like `Owl.Box`.","ref":"Owl.Data.html#from_chardata/1"},{"type":"function","title":"Examples - Owl.Data.from_chardata/1","doc":"iex> [:red, \"hello\"] |> IO.ANSI.format() |> Owl.Data.from_chardata()\n    Owl.Data.tag(\"hello\", :red)\n\n    іex> {output, 0} = Owl.System.cmd(\"bat\", [\"README.md\", \"--color=always\", \"--style=plain\"])\n    ...> output\n    ...> |> Owl.Data.from_chardata()\n    ...> |> Owl.Box.new(title: Owl.Data.tag(\"README.md\", :cyan), border_tag: :light_cyan)\n    ...> |> Owl.IO.puts()","ref":"Owl.Data.html#from_chardata/1-examples"},{"type":"function","title":"Owl.Data.length/1","doc":"Returns length of the data.","ref":"Owl.Data.html#length/1"},{"type":"function","title":"Examples - Owl.Data.length/1","doc":"iex> Owl.Data.length([\"222\"])\n    3\n\n    iex> Owl.Data.length([222])\n    1\n\n    iex> Owl.Data.length([[[]]])\n    0\n\n    iex> Owl.Data.length([\"222\", Owl.Data.tag([\"333\", \"444\"], :green)])\n    9\n\n    # if ucwidth dependency is present, then it is used to calculate the length of the string\n    iex> Owl.Data.length(\"😂\")\n    2","ref":"Owl.Data.html#length/1-examples"},{"type":"function","title":"Owl.Data.lines/1","doc":"Splits data by new lines.\n\nA special case of `split/2`.","ref":"Owl.Data.html#lines/1"},{"type":"function","title":"Example - Owl.Data.lines/1","doc":"iex> Owl.Data.lines([\"first\\nsecond\\n\", Owl.Data.tag(\"third\\nfourth\", :red)])\n    [\"first\", \"second\", Owl.Data.tag([\"third\"], :red), Owl.Data.tag([\"fourth\"], :red)]","ref":"Owl.Data.html#lines/1-example"},{"type":"function","title":"Owl.Data.slice/3","doc":"Returns a data starting at the offset `start`, and of the given `length`.\n\nIt is like `String.slice/3` but for `t:t/0`.","ref":"Owl.Data.html#slice/3"},{"type":"function","title":"Examples - Owl.Data.slice/3","doc":"iex> Owl.Data.slice([Owl.Data.tag(\"Hello world\", :red), Owl.Data.tag(\"!\", :green)], 6, 7)\n    [Owl.Data.tag([\"world\"], :red), Owl.Data.tag([\"!\"], :green)]\n\n    iex> Owl.Data.slice(Owl.Data.tag(\"Hello world\", :red), 20, 10)\n    []","ref":"Owl.Data.html#slice/3-examples"},{"type":"function","title":"Owl.Data.split/2","doc":"Divides data into parts based on a pattern saving sequences for tagged data in new tags.","ref":"Owl.Data.html#split/2"},{"type":"function","title":"Example - Owl.Data.split/2","doc":"iex> Owl.Data.split([\"first second \", Owl.Data.tag(\"third fourth\", :red)], \" \")\n    [\"first\", \"second\", Owl.Data.tag([\"third\"], :red), Owl.Data.tag([\"fourth\"], :red)]\n\n    iex> Owl.Data.split([\"first   second \", Owl.Data.tag(\"third    fourth\", :red)], ~r/ +/)\n    [\"first\", \"second\", Owl.Data.tag([\"third\"], :red), Owl.Data.tag([\"fourth\"], :red)]","ref":"Owl.Data.html#split/2-example"},{"type":"function","title":"Owl.Data.tag/2","doc":"Builds a tag.","ref":"Owl.Data.html#tag/2"},{"type":"function","title":"Examples - Owl.Data.tag/2","doc":"iex> Owl.Data.tag([\"hello \", Owl.Data.tag(\"world\", :green), \"!!!\"], :red)\n    Owl.Data.tag([\"hello \", Owl.Data.tag(\"world\", :green), \"!!!\"], :red)\n\n    iex> Owl.Data.tag(\"hello world\", [:green, :red_background])\n    Owl.Data.tag(\"hello world\", [:green, :red_background])","ref":"Owl.Data.html#tag/2-examples"},{"type":"function","title":"Owl.Data.to_chardata/1","doc":"Transforms data to `t:chardata/0` format which can be consumed by `IO` module.","ref":"Owl.Data.html#to_chardata/1"},{"type":"function","title":"Examples - Owl.Data.to_chardata/1","doc":"iex> \"hello\" |> Owl.Data.tag(:red) |> Owl.Data.to_chardata()\n    [[[[[] | \"\\e[31m\"], \"hello\"] | \"\\e[39m\"] | \"\\e[0m\"]","ref":"Owl.Data.html#to_chardata/1-examples"},{"type":"function","title":"Owl.Data.truncate/2","doc":"Truncates data, so the length of returning data is <= `length`.\n\nPuts ellipsis symbol at the end if data was truncated.","ref":"Owl.Data.html#truncate/2"},{"type":"function","title":"Examples - Owl.Data.truncate/2","doc":"iex> Owl.Data.truncate([Owl.Data.tag(\"Hello\", :red), Owl.Data.tag(\" world!\", :green)], 10)\n    [Owl.Data.tag([\"Hello\"], :red), Owl.Data.tag([\" wor\"], :green), \"…\"]\n\n    iex> Owl.Data.truncate(\"Hello\", 10)\n    \"Hello\"\n\n    iex> Owl.Data.truncate(\"Hello\", 4)\n    [\"Hel\", \"…\"]\n\n    iex> Owl.Data.truncate(\"Hello\", 5)\n    \"Hello\"\n\n    iex> Owl.Data.truncate(\"Hello\", 1)\n    \"…\"","ref":"Owl.Data.html#truncate/2-examples"},{"type":"function","title":"Owl.Data.unlines/1","doc":"Creates a `t:t/0` from an a list of `t:t/0`, it inserts new line characters between original elements.","ref":"Owl.Data.html#unlines/1"},{"type":"function","title":"Examples - Owl.Data.unlines/1","doc":"iex> Owl.Data.unlines([\"a\", \"b\", \"c\"])\n    [\"a\", \"\\n\", \"b\", \"\\n\", \"c\"]\n\n    iex> [\"first\\nsecond\\n\", Owl.Data.tag(\"third\\nfourth\", :red)]\n    ...> |> Owl.Data.lines()\n    ...> |> Owl.Data.unlines()\n    ...> |> Owl.Data.to_chardata()\n    Owl.Data.to_chardata([\"first\\nsecond\\n\", Owl.Data.tag(\"third\\nfourth\", :red)])","ref":"Owl.Data.html#unlines/1-examples"},{"type":"function","title":"Owl.Data.untag/1","doc":"Removes information about sequences and keeps only content of the tag.","ref":"Owl.Data.html#untag/1"},{"type":"function","title":"Examples - Owl.Data.untag/1","doc":"iex> Owl.Data.tag(\"Hello\", :red) |> Owl.Data.untag()\n    \"Hello\"\n\n    iex> Owl.Data.tag([72, 101, 108, 108, 111], :red) |> Owl.Data.untag()\n    ~c\"Hello\"\n\n    iex> Owl.Data.tag([\"Hello\", Owl.Data.tag(\"world\", :green)], :red) |> Owl.Data.untag()\n    [\"Hello\", \"world\"]\n\n    iex> [\"Hello \", Owl.Data.tag(\"world\", :red), [\"!\"]] |> Owl.Data.untag()\n    [\"Hello \", \"world\", [\"!\"]]","ref":"Owl.Data.html#untag/1-examples"},{"type":"function","title":"Owl.Data.zip/2","doc":"Zips corresponding lines into 1 line.\n\nThe zipping finishes as soon as either data completes.","ref":"Owl.Data.html#zip/2"},{"type":"function","title":"Examples - Owl.Data.zip/2","doc":"iex> Owl.Data.zip(\"a\\nb\\nc\", \"d\\ne\\nf\")\n    [[\"a\", \"d\"], \"\\n\", [\"b\", \"e\"], \"\\n\", [\"c\", \"f\"]]\n\n    iex> Owl.Data.zip(\"a\\nb\", \"c\")\n    [[\"a\", \"c\"]]\n\n    iex> 1..3\n    ...> |> Enum.map(&to_string/1)\n    ...> |> Enum.map(&Owl.Box.new/1) |> Enum.reduce(&Owl.Data.zip/2) |> to_string()\n    \"\"\"\n    ┌─┐┌─┐┌─┐\n    │3││2││1│\n    └─┘└─┘└─┘\n    \"\"\" |> String.trim_trailing()","ref":"Owl.Data.html#zip/2-examples"},{"type":"type","title":"Owl.Data.sequence/0","doc":"ANSI escape sequence.\n\nAn atom alias of ANSI escape sequence.\n\nA binary representation of color like `\"\\e[38;5;33m\"` (which is `IO.ANSI.color(33)` or `IO.ANSI.color(0, 2, 5)`).","ref":"Owl.Data.html#t:sequence/0"},{"type":"type","title":"Owl.Data.t/0","doc":"A recursive data type that is similar to `t:chardata/0`, but additionally supports `t:Owl.Tag.t/1`.\n\nCan be printed using `Owl.IO.puts/2`.","ref":"Owl.Data.html#t:t/0"},{"type":"module","title":"Owl.IO","doc":"A set of functions for handling IO with support of `t:Owl.Data.t/0`.","ref":"Owl.IO.html"},{"type":"function","title":"Owl.IO.columns/1","doc":"Returns a width of a terminal.\n\nA wrapper around `:io.columns/1`, but returns `nil` if terminal is not found.\nThis is useful for convenient falling back to other value using `||/2` operator.","ref":"Owl.IO.html#columns/1"},{"type":"function","title":"Example - Owl.IO.columns/1","doc":"Owl.IO.columns() || 80","ref":"Owl.IO.html#columns/1-example"},{"type":"function","title":"Owl.IO.confirm/1","doc":"Asks user to type a confirmation.\n\nValid inputs are a blank string and values specified in `:answers` option.\nUser will be asked to type a confirmation again on invalid input.","ref":"Owl.IO.html#confirm/1"},{"type":"function","title":"Options - Owl.IO.confirm/1","doc":"* `:message` - typically a question about performing operation. Defaults to `\"Are you sure?\"`.\n* `:default` - a value that is used when user responds with a blank string. Defaults to `false`.\n* `:answers` - allows to specify alternative answers. Defaults to `[true: {\"y\", [\"yes\"]}, false: {\"n\", [\"no\"]}]`.","ref":"Owl.IO.html#confirm/1-options"},{"type":"function","title":"Examples - Owl.IO.confirm/1","doc":"Owl.IO.confirm()\n    #=> Are you sure? [yN] n\n    false\n\n    Owl.IO.confirm(message: Owl.Data.tag(\"Really?\", :red), default: true)\n    #=> Really? [Yn]\n    true\n\n    Owl.IO.confirm(\n      message: Owl.Data.tag(\"Справді?\", :red),\n      answers: [true: {\"т\", [\"так\", \"y\", \"yes\"]}, false: {\"н\", [\"ні\", \"n\", \"no\"]}]\n    )\n    #=> Справді? [тН] НІ\n    false","ref":"Owl.IO.html#confirm/1-examples"},{"type":"function","title":"Owl.IO.input/1","doc":"Reads a line from the `stdio` and casts a value to the given type.\n\nAfter reading a line from `stdio` it will be automatically trimmed with `String.trim/2`.\nThe end value will be returned when user types a valid value.","ref":"Owl.IO.html#input/1"},{"type":"function","title":"Options - Owl.IO.input/1","doc":"* `:secret` - set to `true` if you want to make input invisible. Defaults to `false`.\n* `:label` - a text label. Defaults to `nil` (no label).\n* `:optional` - a boolean that sets whether value is optional. Defaults to `false`.\n* `:cast` - casts a value after reading it from `stdio`. Defaults to `:string`. Possible values:\n  * an anonymous function with arity 1 that is described by `t:cast_input/0`\n  * a pair with built-in type represented as atom and a keyword-list with options. Built-in types:\n    * `:integer`, options:\n      * `:min` - a minimum allowed value. Defaults to `nil` (no lower bound).\n      * `:max` - a maximum allowed value. Defaults to `nil` (no upper bound).\n    * `:string`, options:\n      * no options\n  * an atom which is simply an alias to `{atom(), []}`","ref":"Owl.IO.html#input/1-options"},{"type":"function","title":"Examples - Owl.IO.input/1","doc":"Owl.IO.input()\n    #=> > hello world\n    \"hello world\"\n\n    Owl.IO.input(secret: true)\n    #=> >\n    \"password\"\n\n    Owl.IO.input(optional: true)\n    #=> >\n    nil\n\n    Owl.IO.input(label: \"Your age\", cast: {:integer, min: 18, max: 100})\n    #=> Your age\n    #=> > 12\n    #=> must be greater than or equal to 18\n    #=> Your age\n    #=> > 102\n    #=> must be less than or equal to 100\n    #=> Your age\n    #=> > 18\n    18\n\n    Owl.IO.input(label: \"Birth date in ISO 8601 format:\", cast: &Date.from_iso8601/1)\n    #=> Birth date in ISO 8601 format:\n    #=> > 1 January\n    #=> invalid_format\n    #=> Birth date in ISO 8601 format:\n    #=> > 2021-01-01\n    ~D[2021-01-01]","ref":"Owl.IO.html#input/1-examples"},{"type":"function","title":"Owl.IO.inspect/3","doc":"Wrapper around `IO.inspect/3` with changed defaults.\n\nAs in `puts/2`, `device` argument is moved to the end.\nOptions are the same as for `IO.inspect/3` with small changes:\n* `:pretty` is `true` by default.\n* `:syntax_colors` uses color schema from `IEx` by default.\n* `:label` is extended and accepts `t:Owl.Data.t/0`.","ref":"Owl.IO.html#inspect/3"},{"type":"function","title":"Examples - Owl.IO.inspect/3","doc":"\"Hello\"\n    |> Owl.IO.inspect(label: \"Greeting\")\n    |> String.upcase()\n    |> Owl.IO.inspect(label: Owl.Data.tag(\"GREETING\", :cyan))\n    #=> Greeting: \"Hello\"\n    #=> GREETING: \"HELLO\"\n\n    # inspect data above rendered live blocks\n    Owl.IO.inspect(\"Hello\", [], Owl.LiveScreen)\n    #=> \"Hello\"","ref":"Owl.IO.html#inspect/3-examples"},{"type":"function","title":"Owl.IO.multiselect/2","doc":"Select multiple values from the given nonempty list.\n\nInput item numbers must be separated by any non-digit character. Most likely you'd want to use spaces or commas.\nIt is possible to specify a range of numbers using hyphen.","ref":"Owl.IO.html#multiselect/2"},{"type":"function","title":"Options - Owl.IO.multiselect/2","doc":"* `:label` - a text label. Defaults to `nil` (no label).\n* `:render_as` - a function that renders given item. Defaults to `Function.identity/1`.\n* `:min` - a minimum output list length. Defaults to `nil` (no lower bound).\n* `:max` - a maximum output list length. Defaults to `nil` (no upper bound).","ref":"Owl.IO.html#multiselect/2-options"},{"type":"function","title":"Examples - Owl.IO.multiselect/2","doc":"Owl.IO.multiselect([\"one\", \"two\", \"three\"], min: 2, label: \"Select 2 numbers:\", render_as: &String.upcase/1)\n    #=> 1. ONE\n    #=> 2. TWO\n    #=> 3. THREE\n    #=>\n    #=> Select 2 numbers:\n    #=> > 1\n    #=> the number of elements must be greater than or equal to 2\n    #=> Select 2 numbers:\n    #=> > 1 3\n    #=>\n    [\"one\", \"three\"]\n\n    Owl.IO.multiselect(Enum.to_list(1..5), render_as: &to_string/1)\n    #=> 1. 1\n    #=> 2. 2\n    #=> 3. 3\n    #=> 4. 4\n    #=> 5. 5\n    #=>\n    #=> > 1-3 5\n    #=>\n    [1, 2, 3, 5]","ref":"Owl.IO.html#multiselect/2-examples"},{"type":"function","title":"Owl.IO.open_in_editor/2","doc":"Opens `data` in editor for editing.\n\nReturns updated data when file is saved and editor is closed.\nSimilarly to `IEx.Helpers.open/1`, this function uses `ELIXIR_EDITOR` environment variable by default.\n`__FILE__` notation is supported as well.","ref":"Owl.IO.html#open_in_editor/2"},{"type":"function","title":"Example - Owl.IO.open_in_editor/2","doc":"# use neovim in alacritty terminal emulator as an editor\n    $ export ELIXIR_EDITOR=\"alacritty -e nvim\"\n\n    # open editor from Elixir code\n    Owl.IO.open_in_editor(\"hello\\nworld\")\n\n    # specify editor explicitly\n    Owl.IO.open_in_editor(\"hello\\nworld\", \"alacritty -e nvim\")","ref":"Owl.IO.html#open_in_editor/2-example"},{"type":"function","title":"Owl.IO.puts/2","doc":"Wrapper around `IO.puts/2` that accepts `t:Owl.Data.t/0`.\n\nThe other difference is that `device` argument is moved to second argument.","ref":"Owl.IO.html#puts/2"},{"type":"function","title":"Examples - Owl.IO.puts/2","doc":"Owl.IO.puts([\"Hello \", Owl.Data.tag(\"world\", :green)])\n    #=> Hello world\n\n    # specify Owl.LiveScreen as a device in order to print data above rendered live blocks\n    Owl.IO.puts([\"Hello \", Owl.Data.tag(\"world\", :green)], Owl.LiveScreen)\n    #=> Hello world","ref":"Owl.IO.html#puts/2-examples"},{"type":"function","title":"Owl.IO.rows/1","doc":"Returns a height of a terminal.\n\nA wrapper around `:io.rows/1`, but returns `nil` if terminal is not found.\nThis is useful for convenient falling back to other value using `||/2` operator.","ref":"Owl.IO.html#rows/1"},{"type":"function","title":"Example - Owl.IO.rows/1","doc":"Owl.IO.rows() || 20","ref":"Owl.IO.html#rows/1-example"},{"type":"function","title":"Owl.IO.select/2","doc":"Selects one item from the given nonempty list.\n\nReturns value immediately if list contains only 1 element.","ref":"Owl.IO.html#select/2"},{"type":"function","title":"Options - Owl.IO.select/2","doc":"* `:label` - a text label. Defaults to `nil` (no label).\n* `:render_as` - a function that renders given item. Defaults to `Function.identity/1`.","ref":"Owl.IO.html#select/2-options"},{"type":"function","title":"Examples - Owl.IO.select/2","doc":"Owl.IO.select([\"one\", \"two\", \"three\"])\n    #=> 1. one\n    #=> 2. two\n    #=> 3. three\n    #=>\n    #=> > 1\n    #=>\n    \"one\"\n\n\n    ~D[2001-01-01]\n    |> Date.range(~D[2001-01-03])\n    |> Enum.to_list()\n    |> Owl.IO.select(render_as: &Date.to_iso8601/1, label: \"Please select a date\")\n    #=> 1. 2001-01-01\n    #=> 2. 2001-01-02\n    #=> 3. 2001-01-03\n    #=>\n    #=> Please select a date\n    #=> > 2\n    #=>\n    ~D[2001-01-02]\n\n\n    packages = [\n      %{name: \"elixir\", description: \"programming language\"},\n      %{name: \"asdf\", description: \"version manager\"},\n      %{name: \"neovim\", description: \"fork of vim\"}\n    ]\n    Owl.IO.select(packages,\n      render_as: fn %{name: name, description: description} ->\n        [Owl.Data.tag(name, :cyan), \"\\n  \", Owl.Data.tag(description, :light_black)]\n      end\n    )\n    #=> 1. elixir\n    #=>      programming language\n    #=> 2. asdf\n    #=>      version manager\n    #=> 3. neovim\n    #=>      fork of vim\n    #=>\n    #=> > 3\n    #=>\n    %{description: \"fork of vim\", name: \"neovim\"}","ref":"Owl.IO.html#select/2-examples"},{"type":"type","title":"Owl.IO.cast_input/0","doc":"","ref":"Owl.IO.html#t:cast_input/0"},{"type":"type","title":"Owl.IO.confirm_option/0","doc":"","ref":"Owl.IO.html#t:confirm_option/0"},{"type":"type","title":"Owl.IO.input_option/0","doc":"","ref":"Owl.IO.html#t:input_option/0"},{"type":"type","title":"Owl.IO.multiselect_option/0","doc":"","ref":"Owl.IO.html#t:multiselect_option/0"},{"type":"type","title":"Owl.IO.select_option/0","doc":"","ref":"Owl.IO.html#t:select_option/0"},{"type":"module","title":"Owl.LiveScreen","doc":"A server that handles live updates in terminal.\n\nIt partially implements [The Erlang I/O Protocol](https://www.erlang.org/doc/apps/stdlib/io_protocol.html),\nso it is possible to use `Owl.LiveScreen` as an I/O-device in `Logger`\nand functions like `Owl.IO.puts/2`, `IO.puts/2`. When used as I/O-device, then output is printed above dynamic blocks.","ref":"Owl.LiveScreen.html"},{"type":"module","title":"Example - Owl.LiveScreen","doc":"require Logger\n    :ok = :logger.remove_handler(:default)\n\n    :ok =\n      :logger.add_handler(:default, :logger_std_h, %{\n        config: %{type: {:device, Owl.LiveScreen}},\n        formatter: Logger.Formatter.new()\n      })\n\n    Owl.LiveScreen.add_block(:dependency,\n      state: :init,\n      render: fn\n        :init -> \"init...\"\n        dependency -> [\"dependency: \", Owl.Data.tag(dependency, :yellow)]\n      end\n    )\n\n    Owl.LiveScreen.add_block(:compiling,\n      render: fn\n        :init -> \"init...\"\n        filename -> [\"compiling: \", Owl.Data.tag(to_string(filename), :cyan)]\n      end\n    )\n\n    [\"ecto\", \"phoenix\", \"ex_doc\", \"broadway\"]\n    |> Enum.each(fn dependency ->\n      Owl.LiveScreen.update(:dependency, dependency)\n\n      1..5\n      |> Enum.map(&\"filename#{&1}.ex\")\n      |> Enum.each(fn filename ->\n        Owl.LiveScreen.update(:compiling, filename)\n        Process.sleep(1000)\n        Logger.debug(\"#{filename} compiled for dependency #{dependency}\")\n      end)\n    end)","ref":"Owl.LiveScreen.html#module-example"},{"type":"function","title":"Owl.LiveScreen.add_block/3","doc":"Adds a sticky block to the bottom of the screen that can be updated using `update/3`.","ref":"Owl.LiveScreen.html#add_block/3"},{"type":"function","title":"Options - Owl.LiveScreen.add_block/3","doc":"* `:render` - a function that accepts `state` and returns a view of the block. Defaults to `Function.identity/1`, which\nmeans that state has to have type `t:Owl.Data.t/0`.\n* `:state` - initial state of the block. Defaults to `nil`.","ref":"Owl.LiveScreen.html#add_block/3-options"},{"type":"function","title":"Example - Owl.LiveScreen.add_block/3","doc":"Owl.LiveScreen.add_block(:footer, state: \"starting...\")\n    # which is equivalent to\n    Owl.LiveScreen.add_block(:footer, render: fn\n      nil -> \"starting...\"\n      data -> data\n    end)","ref":"Owl.LiveScreen.html#add_block/3-example"},{"type":"function","title":"Owl.LiveScreen.await_render/1","doc":"Awaits for next rendering.\n\nThis is useful when you want to ensure that last published state is rendered on the screen.","ref":"Owl.LiveScreen.html#await_render/1"},{"type":"function","title":"Example - Owl.LiveScreen.await_render/1","doc":"Owl.LiveScreen.add_block(:status, state: \"starting...\")\n    Owl.LiveScreen.update(:status, \"done!\")\n    Owl.LiveScreen.await_render()","ref":"Owl.LiveScreen.html#await_render/1-example"},{"type":"function","title":"Owl.LiveScreen.capture_stdio/2","doc":"Redirects output from `:stdio` to `Owl.LiveScreen`.","ref":"Owl.LiveScreen.html#capture_stdio/2"},{"type":"function","title":"Example - Owl.LiveScreen.capture_stdio/2","doc":"Owl.ProgressBar.start(id: :users, label: \"Creating users\", total: 100)\n\n    Owl.LiveScreen.capture_stdio(fn ->\n      Enum.each(1..100, fn i ->\n        Process.sleep(10)\n        Owl.ProgressBar.inc(id: :users)\n        # Output from next call will be printed above progress bar\n        IO.inspect([:test, i])\n      end)\n    end)\n\n    Owl.LiveScreen.await_render()","ref":"Owl.LiveScreen.html#capture_stdio/2-example"},{"type":"function","title":"Owl.LiveScreen.flush/1","doc":"Renders data in buffer and detaches blocks.","ref":"Owl.LiveScreen.html#flush/1"},{"type":"function","title":"Owl.LiveScreen.start_link/1","doc":"Starts a server.\n\nServer is started automatically by `:owl` application as a named process.","ref":"Owl.LiveScreen.html#start_link/1"},{"type":"function","title":"Options - Owl.LiveScreen.start_link/1","doc":"* `:name` - used for name registration as described in the \"Name\nregistration\" section in the documentation for `GenServer`. Defaults to `Owl.LiveScreen`\n* `:refresh_every` - a period of refreshing a screen in milliseconds. Defaults to 60.\n* `:terminal_width` - a width of terminal in symbols. Defaults to `:auto`, which gets value from `Owl.IO.columns/1`.\n* `:device` - an I/O device. Defaults to `:stdio`.\nIf terminal is not available, then the server won't be started.","ref":"Owl.LiveScreen.html#start_link/1-options"},{"type":"function","title":"Owl.LiveScreen.stop/1","doc":"Renders data in buffer and terminates a server.","ref":"Owl.LiveScreen.html#stop/1"},{"type":"function","title":"Owl.LiveScreen.update/3","doc":"Updates a state of the block for using it in the next render iteration.","ref":"Owl.LiveScreen.html#update/3"},{"type":"function","title":"Example - Owl.LiveScreen.update/3","doc":"Owl.LiveScreen.add_block(:footer, state: \"starting...\")\n    Process.sleep(1000)\n    Owl.LiveScreen.update(:footer, \"...almost done...\")\n    Process.sleep(1000)\n    Owl.LiveScreen.update(:footer, \"done!!!\")","ref":"Owl.LiveScreen.html#update/3-example"},{"type":"type","title":"Owl.LiveScreen.add_block_option/0","doc":"","ref":"Owl.LiveScreen.html#t:add_block_option/0"},{"type":"type","title":"Owl.LiveScreen.block_id/0","doc":"","ref":"Owl.LiveScreen.html#t:block_id/0"},{"type":"type","title":"Owl.LiveScreen.start_option/0","doc":"","ref":"Owl.LiveScreen.html#t:start_option/0"},{"type":"module","title":"Owl.Palette","doc":"Poor man's color picker.","ref":"Owl.Palette.html"},{"type":"function","title":"Owl.Palette.codes/0","doc":"Returns palette with codes from 0 to 255.\n\n    Owl.Palette.codes() |> Owl.IO.puts()\n\nSelected color can be used as follows\n\n    # print \"test\" using foreground color with code 161\n    \"test\" |> Owl.Data.tag(IO.ANSI.color(161)) |> Owl.IO.puts\n\n    # print \"test\" using background color with code 161\n    \"test\" |> Owl.Data.tag(IO.ANSI.color_background(161)) |> Owl.IO.puts","ref":"Owl.Palette.html#codes/0"},{"type":"function","title":"Owl.Palette.named/0","doc":"Returns palette with named codes.\n\n    Owl.Palette.named() |> Owl.IO.puts()\n\nSelected color can be used as follows:\n\n    # print \"test\" using cyan foreground color\n    \"test\" |> Owl.Data.tag(:cyan) |> Owl.IO.puts\n\n    # print \"test\" using light_green foreground color\n    \"test\" |> Owl.Data.tag(:light_green) |> Owl.IO.puts\n\n    # print \"test\" using light_green background color\n    \"test\" |> Owl.Data.tag(:light_green_background) |> Owl.IO.puts\n\n    # print \"test\" using a faint cyan foreground color\n    \"test\" |> Owl.Data.tag([:faint, :cyan]) |> Owl.IO.puts\n\nNote that `:faint` is not supported in all terminals.","ref":"Owl.Palette.html#named/0"},{"type":"function","title":"Owl.Palette.rgb/0","doc":"Returns palette with individual RGB values.\n\n    Owl.Palette.rgb() |> Owl.IO.puts()\n\nSelected color can be used as follows\n\n    # print \"test\" using foreground color RGB(4, 3, 2)\n    \"test\" |> Owl.Data.tag(IO.ANSI.color(4, 3, 2)) |> Owl.IO.puts\n\n    # print \"test\" using background color RGB(4, 3, 2)\n    \"test\" |> Owl.Data.tag(IO.ANSI.color_background(4, 3, 2)) |> Owl.IO.puts","ref":"Owl.Palette.html#rgb/0"},{"type":"module","title":"Owl.ProgressBar","doc":"A live progress bar widget.","ref":"Owl.ProgressBar.html"},{"type":"module","title":"Single bar - Owl.ProgressBar","doc":"Owl.ProgressBar.start(id: :users, label: \"Creating users\", total: 1000)\n\n    Enum.each(1..1000, fn _ ->\n      Process.sleep(10)\n      Owl.ProgressBar.inc(id: :users)\n    end)\n\n    Owl.LiveScreen.await_render()","ref":"Owl.ProgressBar.html#module-single-bar"},{"type":"module","title":"Multiple bars - Owl.ProgressBar","doc":"1..10\n    |> Enum.map(fn index ->\n      Task.async(fn ->\n        range = 1..Enum.random(100..500)\n\n        label = \"Demo Progress ##{index}\"\n\n        Owl.ProgressBar.start(\n          id: {:demo, index},\n          label: label,\n          total: range.last,\n          timer: true,\n          bar_width_ratio: 0.3,\n          filled_symbol: \"#\",\n          partial_symbols: []\n        )\n\n        Enum.each(range, fn _ ->\n          Process.sleep(Enum.random(10..50))\n          Owl.ProgressBar.inc(id: {:demo, index})\n        end)\n      end)\n    end)\n    |> Task.await_many(:infinity)\n\n    Owl.LiveScreen.await_render()","ref":"Owl.ProgressBar.html#module-multiple-bars"},{"type":"function","title":"Owl.ProgressBar.inc/1","doc":"Increases `current` value by `step`.\n\nWhen `current` value becomes equal to `total`, then progress bar terminates.","ref":"Owl.ProgressBar.html#inc/1"},{"type":"function","title":"Options - Owl.ProgressBar.inc/1","doc":"* `:id` - an required identifier of the progress bar.\n* `:step` - a value by which `current` value should be increased. Defaults to 1.","ref":"Owl.ProgressBar.html#inc/1-options"},{"type":"function","title":"Examples - Owl.ProgressBar.inc/1","doc":"Owl.ProgressBar.inc(id: \"Creating users\")\n\n    Owl.ProgressBar.inc(id: \"Creating users\", step: 10)","ref":"Owl.ProgressBar.html#inc/1-examples"},{"type":"function","title":"Owl.ProgressBar.render/1","doc":"Renders a progress bar that can be consumed by `Owl.IO.puts/2`.\n\nUsed as a callback for blocks in `Owl.LiveScreen`.","ref":"Owl.ProgressBar.html#render/1"},{"type":"function","title":"Examples - Owl.ProgressBar.render/1","doc":"iex> Owl.ProgressBar.render(%{\n    ...>   label: \"Demo\",\n    ...>   total: 200,\n    ...>   current: 60,\n    ...>   bar_width_ratio: 0.7,\n    ...>   start_symbol: \"[\",\n    ...>   end_symbol: \"]\",\n    ...>   filled_symbol: \"#\",\n    ...>   partial_symbols: [],\n    ...>   empty_symbol: \".\",\n    ...>   screen_width: 40\n    ...> }) |> to_string()\n    \"Demo      [######.................]  30%\"\n\n    iex> Owl.ProgressBar.render(%{\n    ...>   label: \"Demo\",\n    ...>   total: 200,\n    ...>   current: 8,\n    ...>   bar_width_ratio: 0.2,\n    ...>   start_symbol: \"|\",\n    ...>   absolute_values: true,\n    ...>   end_symbol: \"|\",\n    ...>   filled_symbol: \"█\",\n    ...>   partial_symbols: [\"▏\", \"▎\", \"▍\", \"▌\", \"▋\", \"▊\", \"▉\"],\n    ...>   empty_symbol: \" \",\n    ...>   screen_width: 40,\n    ...>   start_time: -576460748012758993,\n    ...>   current_time: -576460748012729828\n    ...> }) |> to_string()\n    \"Demo         8/200 00:29.2 |▎     |   4%\"\n\n    iex> Owl.ProgressBar.render(%{\n    ...>   label: \"Demo\",\n    ...>   total: 200,\n    ...>   current: 8,\n    ...>   bar_width_ratio: 0.7,\n    ...>   start_symbol: \"[\",\n    ...>   end_symbol: \"]\",\n    ...>   filled_symbol: Owl.Data.tag(\"≡\", :cyan),\n    ...>   partial_symbols: [Owl.Data.tag(\"-\", :green), Owl.Data.tag(\"=\", :blue)],\n    ...>   empty_symbol: \" \",\n    ...>   screen_width: 40\n    ...> })|> Owl.Data.to_chardata() |> to_string()\n    \"Demo      [\u001B[34m=\u001B[39m                      ]   4%\u001B[0m\"\n\n    iex> Owl.ProgressBar.render(%{\n    ...>   label: \"Demo\",\n    ...>   total: 200,\n    ...>   current: 60,\n    ...>   bar_width_ratio: 0.95,\n    ...>   start_symbol: \"[\",\n    ...>   end_symbol: \"]\",\n    ...>   filled_symbol: \"#\",\n    ...>   partial_symbols: [],\n    ...>   empty_symbol: \".\",\n    ...>   screen_width: 40\n    ...> }) |> to_string()\n    \"D…[#########......................]  30%\"\n\n    iex> Owl.ProgressBar.render(%{\n    ...>   label: \"Demo\",\n    ...>   total: 200,\n    ...>   current: 60,\n    ...>   bar_width_ratio: 1,\n    ...>   start_symbol: \"[\",\n    ...>   end_symbol: \"]\",\n    ...>   filled_symbol: \"#\",\n    ...>   partial_symbols: [],\n    ...>   empty_symbol: \".\",\n    ...>   screen_width: 40\n    ...> }) |> to_string()\n    \"[#########........................]  30%\"","ref":"Owl.ProgressBar.html#render/1-examples"},{"type":"function","title":"Owl.ProgressBar.start/1","doc":"Starts a progress bar on `Owl.LiveScreen`.","ref":"Owl.ProgressBar.html#start/1"},{"type":"function","title":"Options - Owl.ProgressBar.start/1","doc":"* `:id` - an id of the progress bar. Required.\n* `:label` - a label of the progress bar. Required.\n* `:total` - a total value. Required.\n* `:current` - a current value. Defaults to `0`.\n* `:bar_width_ratio` - a bar width ratio. Defaults to 0.7.\n* `:timer` - set to `true` to launch a timer and display it before the bar in format `MM:SS`. Defaults to `false`.\n* `:absolute_values` - set to `true` to show absolute values before the bar in format `current/total`. Defaults to `false`.\n* `:start_symbol` - a symbol that is rendered at the beginning of the progress bar. Defaults to `\"[\"`.\n* `:end_symbol` - a symbol that rendered at the end of the progress bar. Defaults to `\"]\"`.\n* `:filled_symbol` - a symbol that use used when `current` value is big enough to fill the cell. Defaults to `\"≡\"`\n* `:partial_symbols` - a list of symbols that are used when `current` value is too small to render\n`filled_symbol`. Defaults to `[\"-\", \"=\"]`.\n* `:empty_symbol` - an empty symbol. Defaults to `\" \"`.\n* `:screen_width` - a width of output data. Defaults to width of the terminal or 80 symbols, if a terminal is not available.\n* `:live_screen_server` - a reference to `Owl.LiveScreen` server. Defaults to `Owl.LiveScreen`.","ref":"Owl.ProgressBar.html#start/1-options"},{"type":"type","title":"Owl.ProgressBar.id/0","doc":"","ref":"Owl.ProgressBar.html#t:id/0"},{"type":"module","title":"Owl.Spinner","doc":"A spinner widget.\n\nSimply run any long-running task using `run/2`:\n\n    Owl.Spinner.run(\n      fn -> Process.sleep(5_000) end,\n      labels: [ok: \"Done\", error: \"Failed\", processing: \"Please wait...\"]\n    )\n\nMultiple spinners can be run simultaneously:\n\n    long_running_tasks =\n      Enum.map([9000, 8000, 4000, 6000], fn delay ->\n        fn -> Process.sleep(delay) end\n      end)\n\n    long_running_tasks\n    |> Task.async_stream(&Owl.Spinner.run/1, timeout: :infinity)\n    |> Stream.run()\n\nMultiline frames are supported as well:\n\n    Owl.Spinner.run(fn -> Process.sleep(5_000) end,\n      frames: [\n        processing: [\n          \"╔════╤╤╤╤════╗\\n║    │││ \\\\   ║\\n║    │││  O  ║\\n║    OOO     ║\",\n          \"╔════╤╤╤╤════╗\\n║    ││││    ║\\n║    ││││    ║\\n║    OOOO    ║\",\n          \"╔════╤╤╤╤════╗\\n║   / │││    ║\\n║  O  │││    ║\\n║     OOO    ║\",\n          \"╔════╤╤╤╤════╗\\n║    ││││    ║\\n║    ││││    ║\\n║    OOOO    ║\"\n        ]\n      ]\n    )","ref":"Owl.Spinner.html"},{"type":"module","title":"Where can I get alternative frames? - Owl.Spinner","doc":"* https://github.com/blackode/elixir_cli_spinners/blob/master/lib/cli_spinners/spinners.ex\n* https://www.google.com/search?q=ascii+spinners","ref":"Owl.Spinner.html#module-where-can-i-get-alternative-frames"},{"type":"function","title":"Owl.Spinner.run/2","doc":"Runs a spinner during execution of `process_function` and returns its result.\n\nThe spinner is started, and automatically stopped after the function returns, regardless if there was an error when executing the function.\nIt is a wrapper around `start/1` and `stop/1`. The only downside of `run/2` is that it is not possible to update\na label while `process_function` is executing.\n\nIf function returns `:ok` or `{:ok, value}` then spinner will be stopped with `:ok` resolution.\n\nIf function returns `:error` or `{:error, reason}` then spinner will be stopped with `:error` resolution.","ref":"Owl.Spinner.html#run/2"},{"type":"function","title":"Options - Owl.Spinner.run/2","doc":"* `:refresh_every` - period of changing frames. Defaults to `100`.\n* `:frames` - allows to set frames for different states of spinner:\n  * `:processing` - list of frames which are rendered until spinner is stopped.\n  Defaults to `[\"⠋\", \"⠙\", \"⠹\", \"⠸\", \"⠼\", \"⠴\", \"⠦\", \"⠧\", \"⠇\", \"⠏\"]`.\n  * `:ok` - frame that is rendered when spinner is stopped with `:ok` resolution.\n  Defaults to `Owl.Data.tag(\"✔\", :green)`.\n  * `:error` - frame that is rendered when spinner is stopped with `:error` resolution.\n  Defaults to `Owl.Data.tag(\"✖\", :red)`.\n* `:labels` - allows to set labels for different states of spinner:\n  * `:processing` - label that is rendered during processing. Cannot be changed during execution of `process_function`.\n  Defaults to `nil`.\n  * `:ok` - label that is rendered when spinner is stopped with `:ok` resolution. A function with arity 1 can be\n  passed in order to format a label based on result of `process_function`.\n  Defaults to `nil`.\n  * `:error` - label that is rendered when spinner is stopped with `:error` resolution. A function with arity 1\n  can be passed in order to format a label based on result of `process_function`.\n  Defaults to `nil`.\n* `:live_screen_server` - a reference to `Owl.LiveScreen` server. Defaults to `Owl.LiveScreen`.","ref":"Owl.Spinner.html#run/2-options"},{"type":"function","title":"Examples - Owl.Spinner.run/2","doc":"Owl.Spinner.run(fn -> Process.sleep(5_000) end)\n    => :ok\n\n    Owl.Spinner.run(fn -> Process.sleep(5_000) end,\n      frames: [\n        # an ASCII fish going back and forth\n        processing: [\n          \">))'>\",\n          \"    >))'>\",\n          \"        >))'>\",\n          \"    <'((<\",\n          \"<'((<\"\n        ]\n      ]\n    )\n    => :ok\n\n    Owl.Spinner.run(\n      fn ->\n        Process.sleep(5_000)\n        {:error, :oops}\n      end,\n      labels: [\n        error: fn reason -> \"Failed: #{inspect(reason)}\" end,\n        processing: \"Processing...\"\n      ]\n    )\n    => {:error, :oops}","ref":"Owl.Spinner.html#run/2-examples"},{"type":"function","title":"Owl.Spinner.start/1","doc":"Starts a new spinner.\n\nMust be stopped manually by calling `stop/1`.","ref":"Owl.Spinner.html#start/1"},{"type":"function","title":"Options - Owl.Spinner.start/1","doc":"* `:id` - an id of the spinner. Required.\n* `:refresh_every` - period of changing frames. Defaults to `100`.\n* `:frames` - allows to set frames for different states of spinner:\n  * `:processing` - list of frames which are rendered until spinner is stopped.\n  Defaults to `[\"⠋\", \"⠙\", \"⠹\", \"⠸\", \"⠼\", \"⠴\", \"⠦\", \"⠧\", \"⠇\", \"⠏\"]`.\n  * `:ok` - frame that is rendered when spinner is stopped with `:ok` resolution.\n  Defaults to `Owl.Data.tag(\"✔\", :green)`.\n  * `:error` - frame that is rendered when spinner is stopped with `:error` resolution.\n  Defaults to `Owl.Data.tag(\"✖\", :red)`.\n* `:labels` - allows to set labels for different states of spinner:\n  * `:processing` - label that is rendered during processing. Can be changed with `update_label/1`.\n  Defaults to `nil`.\n  * `:ok` - label that is rendered when spinner is stopped with `:ok` resolution.\n  Defaults to `nil`.\n  * `:error` - label that is rendered when spinner is stopped with `:error` resolution.\n  Defaults to `nil`.\n* `:live_screen_server` - a reference to `Owl.LiveScreen` server. Defaults to `Owl.LiveScreen`.","ref":"Owl.Spinner.html#start/1-options"},{"type":"function","title":"Example - Owl.Spinner.start/1","doc":"Owl.Spinner.start(id: :my_spinner)\n    Process.sleep(1000)\n    Owl.Spinner.stop(id: :my_spinner, resolution: :ok)","ref":"Owl.Spinner.html#start/1-example"},{"type":"function","title":"Owl.Spinner.stop/1","doc":"Stops the spinner.","ref":"Owl.Spinner.html#stop/1"},{"type":"function","title":"Options - Owl.Spinner.stop/1","doc":"* `:id` - an id of the spinner. Required.\n* `:resolution` - an atom `:ok` or `:error`. Determines frame and label for final rendering. Required.\n* `:label` - a label for final rendering. If not set, then values that are set on spinner start will be used.","ref":"Owl.Spinner.html#stop/1-options"},{"type":"function","title":"Example - Owl.Spinner.stop/1","doc":"Owl.Spinner.stop(id: :my_spinner, resolution: :ok)","ref":"Owl.Spinner.html#stop/1-example"},{"type":"function","title":"Owl.Spinner.update_label/1","doc":"Updates a label of the running spinner.\n\nOverrides a value that is set for `:processing` state on start.","ref":"Owl.Spinner.html#update_label/1"},{"type":"function","title":"Options - Owl.Spinner.update_label/1","doc":"* `:id` - an id of the spinner. Required.\n* `:label` - a new value of the label. Required.","ref":"Owl.Spinner.html#update_label/1-options"},{"type":"function","title":"Example - Owl.Spinner.update_label/1","doc":"Owl.Spinner.start(id: :my_spinner)\n    Owl.Spinner.update_label(id: :my_spinner, label: \"Downloading files...\")\n    Process.sleep(1000)\n    Owl.Spinner.update_label(id: :my_spinner, label: \"Checking signatures...\")\n    Process.sleep(1000)\n    Owl.Spinner.stop(id: :my_spinner, resolution: :ok, label: \"Done\")","ref":"Owl.Spinner.html#update_label/1-example"},{"type":"type","title":"Owl.Spinner.frame/0","doc":"","ref":"Owl.Spinner.html#t:frame/0"},{"type":"type","title":"Owl.Spinner.id/0","doc":"","ref":"Owl.Spinner.html#t:id/0"},{"type":"type","title":"Owl.Spinner.label/0","doc":"","ref":"Owl.Spinner.html#t:label/0"},{"type":"module","title":"Owl.System","doc":"An alternative to some `System` functions.","ref":"Owl.System.html"},{"type":"function","title":"Owl.System.cmd/3","doc":"A wrapper around `System.cmd/3` which additionally logs executed `command`, `args` and `env`.\n\nIf URL is found in logged message, then password in it is masked with asterisks.\nAdditionally, it is possible to explicitly mark env values and arguments as secret for safe logging.\nSee examples for details.","ref":"Owl.System.html#cmd/3"},{"type":"function","title":"Examples - Owl.System.cmd/3","doc":"> Owl.System.cmd(\"echo\", [\"test\"])\n    # 10:25:34.252 [debug] $ echo test\n    {\"test\\n\", 0}\n\n    # marking an argument as secret\n    > Owl.System.cmd(\"echo\", [\"hello\", secret: \"world\"])\n    # 10:25:40.516 [debug] $ echo hello ********\n    {\"hello world\\n\", 0}\n\n    # marking a part of an argument as secret\n    > Owl.System.cmd(\"echo\", [\"hello\", [\"--password=\", {:secret, \"world\"}]])\n    # 10:25:40.516 [debug] $ echo hello --password=********\n    {\"hello --password=world\\n\", 0}\n\n    # marking a env as secret\n    > Owl.System.cmd(\"echo\", [\"hello\", \"world\"], env: [{\"PASSWORD\", {:secret, \"mypassword\"}}])\n    # 10:25:40.516 [debug] $ PASSWORD=******** sh -c \"echo hello world\"\n    {\"hello world\\n\", 0}\n\n    > Owl.System.cmd(\"psql\", [\"postgresql://postgres:postgres@127.0.0.1:5432\", \"-tAc\", \"SELECT 1;\"])\n    # 10:25:50.947 [debug] $ psql postgresql://postgres:********@127.0.0.1:5432 -tAc 'SELECT 1;'\n    {\"1\\n\", 0}","ref":"Owl.System.html#cmd/3-examples"},{"type":"function","title":"Owl.System.daemon_cmd/4","doc":"Runs `command` as a daemon, executes `operation` and kills the daemon afterwards.\n\nAutomatically puts messages from `stderr` and `stdout` to `device` prepending them with `prefix`.\nReturns result of invoking `operation`.","ref":"Owl.System.html#daemon_cmd/4"},{"type":"function","title":"Options - Owl.System.daemon_cmd/4","doc":"* `:prefix` - a prefix for `stderr` and `stdout` messages from daemon. Defaults to `command` followed by colon.\n* `:device` - device to which messages from `stderr` and `stdout` are put. Defaults to `:stdio`.\n* `:ready_check` - a function which checks the content of the messages produced by `command` before writing to `device`.\n  If the function is set, then the execution of the `operation` will be blocked until `ready_check` returns `true`.\n  By default this check is absent and `operation` is invoked immediately without awaiting any message.\n* `:env` - a list of tuples containing environment key-value. The behaviour is similar to the option described in `cmd/3`","ref":"Owl.System.html#daemon_cmd/4-options"},{"type":"function","title":"Example - Owl.System.daemon_cmd/4","doc":"ex> Owl.System.daemon_cmd(\"ping\", [\"8.8.8.8\"], fn ->\n    ..>   Process.sleep(3_000)\n    ..>   2 + 2\n    ..> end)\n    # 00:36:33.963 [debug] $ ping 8.8.8.8\n    #\n    # 00:36:33.964 [debug] Started daemon ping with OS pid 576077\n    # ping:  PING 8.8.8.8 (8.8.8.8) 56(84) bytes of data.\n    # ping:  64 bytes from 8.8.8.8: icmp_seq=1 ttl=118 time=28.3 ms\n    # ping:  64 bytes from 8.8.8.8: icmp_seq=2 ttl=118 time=26.9 ms\n    # ping:  64 bytes from 8.8.8.8: icmp_seq=3 ttl=118 time=28.6 ms\n\n    # 00:36:36.965 [debug] $ kill 576077\n    4\n\n    ex> Owl.System.daemon_cmd(\n    ..>   \"kubectl\",\n    ..>   [\n    ..>     \"port-forward\",\n    ..>     \"--namespace=my-app\",\n    ..>     \"--kubeconfig\",\n    ..>     \"~/.kube/myapp\",\n    ..>     \"my-pod\",\n    ..>     \"5432:5432\"\n    ..>   ],\n    ..>   fn ->\n    ..>     Logger.debug(\"Dump DB\")\n    ..>   end,\n    ..>   prefix: \"kubectl(my-pod): \",\n    ..>   ready_check: &String.contains?(&1, \"Forwarding from\")\n    ..> )\n    # 12:08:01.382 [debug] $ kubectl port-forward --namespace=my-app --kubeconfig ~/.kube/myapp my-pod 5432:5432\n    # 12:08:01.384 [debug] Started daemon kubectl with OS pid 166378\n    # kubectl(my-pod): Forwarding from 127.0.0.1:5432 -> 5432\n    # kubectl(my-pod): Forwarding from [::1]:5432 -> 5432\n    # 12:08:02.484 [debug] Dump DB\n    # 12:08:02.584 [debug] $ kill 166378\n    :ok","ref":"Owl.System.html#daemon_cmd/4-example"},{"type":"function","title":"Owl.System.shell/2","doc":"A wrapper around `System.shell/2` which additionally logs executed `command`.\n\nSimilarly to `cmd/3`, it automatically hides password in found URLs and allows manual hiding of env values.","ref":"Owl.System.html#shell/2"},{"type":"function","title":"Examples - Owl.System.shell/2","doc":"> Owl.System.shell(\"echo hello world\")\n    # 22:36:01.440 [debug] $ sh -c \"echo hello world\"\n    {\"hello world\\n\", 0}\n\n    > Owl.System.shell(\"echo postgresql://postgres:postgres@127.0.0.1:5432\")\n    # 22:36:51.797 [debug] $ sh -c \"echo postgresql://postgres:********@127.0.0.1:5432\"\n    {\"postgresql://postgres:postgres@127.0.0.1:5432\\n\", 0}\n\n    > Owl.System.shell(\"echo $PASSWORD $USERNAME\", env: [{\"USERNAME\", \"john\"}, {\"PASSWORD\", {:secret, \"qwerty\"}}])\n    # 12:38:27.704 [debug] $ USERNAME=john PASSWORD=******** sh -c \"echo \\$PASSWORD \\$USERNAME\"\n    {\"qwerty john\\n\", 0}","ref":"Owl.System.html#shell/2-examples"},{"type":"module","title":"Owl.Table","doc":"Allows drawing awesome tables.","ref":"Owl.Table.html"},{"type":"function","title":"Owl.Table.new/2","doc":"Draws a table.\n\nAccepts a list of maps, where each map represents a row.\nThe keys and values of maps should have the type `t:Owl.Data.t/0`, otherwise use `:render_cell` option to make values printable.","ref":"Owl.Table.html#new/2"},{"type":"function","title":"Options - Owl.Table.new/2","doc":"* `:border_style` - sets the border style. Defaults to `:solid`.\n* `:divide_body_rows` - specifies whether to show divider between rows in body. It is better to use it if cells have multiline values. Ignored, if `:border_style` is set to `:none`. Defaults to `false`.\n* `:filter_columns` - sets a function which filters column (second argument for `Enum.filter/2`). No filter function by default.\n* `:padding_x`- sets horizontal padding. Defaults to `0`.\n* `:render_cell` - sets how to render header and body cells. Accepts either a function or a keyword list. Defaults to `&Function.identity/1`.\nOptions in case of a keyword list:\n  * `:header` - sets a function to render header cell. Defaults to `&Function.identity/1`.\n  * `:body` - sets a function to render body cell. Defaults to `&Function.identity/1`.\n* `:sort_columns` - sets a sorter (second argument for `Enum.sort/2`) for columns. Defaults to `:asc`.\n* `:max_column_widths` - sets max width for columns in symbols. Accepts a function that returns an inner width (content + padding) for each column. Defaults to `fn _ -> :infinity end`.\n* `:max_width` - sets a maximum width of of the table in symbols including borders. Defaults to width of the terminal or `:infinity`, if a terminal is not available.\n* `:word_wrap` - sets the word wrapping mode. Can be `:break_word` or `:normal`. Defaults to `:break_word`. Ignored if `:truncate_lines` is `true`.\n* `:truncate_lines` - specifies whether to truncate lines when they reach width specified by `:max_content_width`. Defaults to `false`.","ref":"Owl.Table.html#new/2-options"},{"type":"function","title":"Examples - Owl.Table.new/2","doc":"# render as is without options\n    iex> [\n    ...>   %{\"id\" => \"1\", \"name\" => \"Yaroslav\"},\n    ...>   %{\"id\" => \"2\", \"name\" => \"Volodymyr\"}\n    ...> ] |> Owl.Table.new() |> to_string()\n    \"\"\"\n    ┌──┬─────────┐\n    │id│name     │\n    ├──┼─────────┤\n    │1 │Yaroslav │\n    │2 │Volodymyr│\n    └──┴─────────┘\n    \"\"\" |> String.trim_trailing()\n\n    # ...and more complex example with a bunch of options\n    iex> [\n    ...>   %{a: :qwertyuiop, b: :asdfghjkl},\n    ...>   %{a: :zxcvbnm, b: :dcb}\n    ...> ]\n    ...> |> Owl.Table.new(\n    ...>   render_cell: [\n    ...>     header: &(&1 |> inspect() |> Owl.Data.tag(:red)),\n    ...>     body: &(&1 |> inspect() |> Owl.Data.truncate(8) |> Owl.Data.tag(:yellow))\n    ...>   ],\n    ...>   divide_body_rows: true,\n    ...>   border_style: :solid_rounded,\n    ...>   padding_x: 1,\n    ...>   sort_columns: :desc\n    ...> )\n    ...> |> Owl.Data.to_chardata()\n    ...> |> to_string()\n    \"\"\"\n    ╭──────────┬──────────╮\n    │ \\e[31m:b\\e[39m       │ \\e[31m:a\\e[39m       │\n    ├──────────┼──────────┤\n    │ \\e[33m:asdfgh…\\e[39m │ \\e[33m:qwerty…\\e[39m │\n    ├──────────┼──────────┤\n    │ \\e[33m:dcb\\e[39m     │ \\e[33m:zxcvbnm\\e[39m │\n    ╰──────────┴──────────╯\\e[0m\n    \"\"\" |> String.trim_trailing()","ref":"Owl.Table.html#new/2-examples"},{"type":"module","title":"Owl.Tag","doc":"A tag struct.\n\nUse `Owl.Data.tag/2` to build a tag.\n\nTag is a container for data and ANSI sequences associated with it.\nIt allows having local binding for styles in console, similar to tags in HTML.\n\nLet's say you have a string that should be written to stdout in red color.\nThis can be easily done by using a naive approach:\n\n    substring = \"world\"\n    IO.puts([IO.ANSI.red(), \"Hello #{substring}!!\"])\n\nIt works well for easy cases. If you want to make `substring` in another color you can try this:\n\n    substring = [IO.ANSI.green(), \"world\"]\n    IO.puts([IO.ANSI.red(), \"Hello #{substring}!!\"])\n\nbut you'll notice, that the text after `substring` is green too. In order make `\"!!\"` part red again, you have\nto write color explicitly:\n\n    IO.puts([IO.ANSI.red(), \"Hello #{substring}#{IO.ANSI.red()}!!\"])\n\nif substring changes background color, you have to return to the previous one too:\n\n    substring = [IO.ANSI.green(), IO.ANSI.red_background() \"world\"]\n    IO.puts([IO.ANSI.red(), \"Hello #{substring}#{[IO.ANSI.red(), IO.ANSI.default_background()]}!!\"])\n\nSuch code is very hard to maintain.\nThis is how the issue can be addressed with `Owl.Data.tag/2`:\n\n    substring = Owl.Data.tag(\"world\", :green)\n    Owl.IO.puts(Owl.Data.tag([\"Hello \", substring, \"!!\"], :red))\n\n    substring = Owl.Data.tag(\"world\", [:green, :red_background])\n    Owl.IO.puts(Owl.Data.tag([\"Hello \", substring, \"!!\"], :red))","ref":"Owl.Tag.html"},{"type":"function","title":"Owl.Tag.new/2","doc":"Use `Owl.Data.tag/2` instead","ref":"Owl.Tag.html#new/2"},{"type":"type","title":"Owl.Tag.t/1","doc":"","ref":"Owl.Tag.html#t:t/1"},{"type":"module","title":"Owl.Task","doc":"Run task using internal supervision tree.","ref":"Owl.Task.html"},{"type":"function","title":"Owl.Task.run/1","doc":"Runs a function as a task under supervision tree.\n\nThis is useful when you need graceful shutdown in simple scripts, where Owl is installed\nusing `Mix.install/2`.","ref":"Owl.Task.html#run/1"},{"type":"function","title":"Example - Owl.Task.run/1","doc":"# content of ping.exs file, run as `elixir --no-halt ping.exs`\n    Mix.install([:owl])\n\n    Owl.Task.run(fn ->\n      Owl.System.daemon_cmd(\"ping\", [\"8.8.8.8\"], fn ->\n        Process.sleep(3000)\n        2 + 2\n      end)\n    end\n\n    System.stop()","ref":"Owl.Task.html#run/1-example"},{"type":"module","title":"Owl.TrueColor","doc":"A module for true color escape sequences","ref":"Owl.TrueColor.html"},{"type":"module","title":"Example - Owl.TrueColor","doc":"\"Hello\"\n    |> Owl.Data.tag([Owl.TrueColor.color(1, 244, 74), Owl.TrueColor.color_background(133, 48, 100)])\n    |> Owl.IO.puts()","ref":"Owl.TrueColor.html#module-example"},{"type":"function","title":"Owl.TrueColor.color/3","doc":"Returns a true color foreground escape sequence for the given RGB values.","ref":"Owl.TrueColor.html#color/3"},{"type":"function","title":"Example - Owl.TrueColor.color/3","doc":"iex> Owl.TrueColor.color(1, 244, 74)\n    \"\\e[38;2;1;244;74m\"","ref":"Owl.TrueColor.html#color/3-example"},{"type":"function","title":"Owl.TrueColor.color_background/3","doc":"Returns a true color background escape sequence for the given RGB values.","ref":"Owl.TrueColor.html#color_background/3"},{"type":"function","title":"Example - Owl.TrueColor.color_background/3","doc":"iex> Owl.TrueColor.color_background(133, 48, 100)\n    \"\\e[48;2;133;48;100m\"","ref":"Owl.TrueColor.html#color_background/3-example"},{"type":"extras","title":"README","doc":"# Owl\n[![CI Status](https://github.com/fuelen/owl/actions/workflows/ci.yml/badge.svg)](https://github.com/fuelen/owl/actions)\n[![Hex.pm](https://img.shields.io/hexpm/v/owl.svg)](https://hex.pm/packages/owl)\n[![Coverage Status](https://coveralls.io/repos/github/fuelen/owl/badge.svg?branch=main)](https://coveralls.io/github/fuelen/owl?branch=main)\n```\n   ,_,\n  {o,o}\n  /)  )\n---\"-\"--\n```\nOwl is a toolkit for writing command-line user interfaces in Elixir.\n\nIt provides a convenient interface for\n\n* colorizing text using tags\n* input control with validations and casting to various data types\n* select/multiselect controls, inspired by AUR package managers\n* tables\n* editing text in `ELIXIR_EDITOR`\n* wrapping multiline colorized data into ASCII boxes\n* printing palette colors\n* progress bars and spinners, multiple bars/spinners at the same time are supported as well\n* live-updating of multiline blocks\n* working with virtual device which partially implements\n[The Erlang I/O Protocol](https://www.erlang.org/doc/apps/stdlib/io_protocol.html) and doesn't conflict with live blocks.","ref":"readme.html"},{"type":"extras","title":"Demo - README","doc":"[![asciicast](https://asciinema.org/a/vOL2PtAEWB88S9G93Iojwprj2.svg)](https://asciinema.org/a/vOL2PtAEWB88S9G93Iojwprj2)\n\nThe code can be found  in [examples](https://github.com/fuelen/owl/tree/main/examples) directory.","ref":"readme.html#demo"},{"type":"extras","title":"Installation - README","doc":"The package can be installed by adding `owl` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:owl, \"~> 0.11\"},\n    # ucwidth is an optional dependency, uncomment it for multibyte characters support (emoji, etc)\n    # {:ucwidth, \"~> 0.2\"}\n  ]\nend\n```\nDocumentation can be found at [https://hexdocs.pm/owl](https://hexdocs.pm/owl).","ref":"readme.html#installation"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}